#include <MsTimer2.h>
#include <Servo.h>

Servo s1, s2, s3, s4, s5;  //control leg angle
Servo s11, s12, s13, s14, s15;  //control leg length

int servo_angle_offset[5] = { -40, 0, 30, 30, 0};
int servo_length_offset[5] = {0, 0, 0, 0, 0};
int move_angle = 500;
int state = 0;
int frame = 0;
int command = 0;  //0=idle, 1=forward, 2=backward, 3=turn left, 4=turn right

struct leg {
  int leg_angle_us;
  int leg_length_us;
};
leg CCW_stride_frame[21] = {1500 + 10 * move_angle / 2 / 10, 1500, //stride forward without touching ground
                            1500 + 9 * move_angle / 2 / 10, 2000,
                            1500 + 8 * move_angle / 2 / 10, 2000,
                            1500 + 7 * move_angle / 2 / 10, 2000,
                            1500 + 6 * move_angle / 2 / 10, 2000,
                            1500 + 5 * move_angle / 2 / 10, 2000,
                            1500 + 4 * move_angle / 2 / 10, 2000,
                            1500 + 3 * move_angle / 2 / 10, 2000,
                            1500 + 2 * move_angle / 2 / 10, 2000,
                            1500 + 1 * move_angle / 2 / 10, 2000,
                            1500, 2000,
                            1500 - 1 * move_angle / 2 / 10, 2000,
                            1500 - 2 * move_angle / 2 / 10, 2000,
                            1500 - 3 * move_angle / 2 / 10, 2000,
                            1500 - 4 * move_angle / 2 / 10, 2000,
                            1500 - 5 * move_angle / 2 / 10, 2000,
                            1500 - 6 * move_angle / 2 / 10, 2000,
                            1500 - 7 * move_angle / 2 / 10, 2000,
                            1500 - 8 * move_angle / 2 / 10, 2000,
                            1500 - 9 * move_angle / 2 / 10, 2000,
                            1500 - 10 * move_angle / 2 / 10, 1500
                           };
leg CW_stride_frame[21] = {1500 - 10 * move_angle / 2 / 10, 1500, //stride backward without touching ground
                           1500 - 9 * move_angle / 2 / 10, 2000,
                           1500 - 8 * move_angle / 2 / 10, 2000,
                           1500 - 7 * move_angle / 2 / 10, 2000,
                           1500 - 6 * move_angle / 2 / 10, 2000,
                           1500 - 5 * move_angle / 2 / 10, 2000,
                           1500 - 4 * move_angle / 2 / 10, 2000,
                           1500 - 3 * move_angle / 2 / 10, 2000,
                           1500 - 2 * move_angle / 2 / 10, 2000,
                           1500 - 1 * move_angle / 2 / 10, 2000,
                           1500, 2000,
                           1500 + 1 * move_angle / 2 / 10, 2000,
                           1500 + 2 * move_angle / 2 / 10, 2000,
                           1500 + 3 * move_angle / 2 / 10, 2000,
                           1500 + 4 * move_angle / 2 / 10, 2000,
                           1500 + 5 * move_angle / 2 / 10, 2000,
                           1500 + 6 * move_angle / 2 / 10, 2000,
                           1500 + 7 * move_angle / 2 / 10, 2000,
                           1500 + 8 * move_angle / 2 / 10, 2000,
                           1500 + 9 * move_angle / 2 / 10, 2000,
                           1500 + 10 * move_angle / 2 / 10, 1500
                          };
leg CCW_kick_frame[21] = {1500 - 10 * move_angle / 2 / 10, 1500, //kick back with touching ground
                          1500 - 9 * move_angle / 2 / 10, 1550,
                          1500 - 8 * move_angle / 2 / 10, 1600,
                          1500 - 7 * move_angle / 2 / 10, 1650,
                          1500 - 6 * move_angle / 2 / 10, 1650,
                          1500 - 5 * move_angle / 2 / 10, 1650,
                          1500 - 4 * move_angle / 2 / 10, 1650,
                          1500 - 3 * move_angle / 2 / 10, 1650,
                          1500 - 2 * move_angle / 2 / 10, 1650,
                          1500 - 1 * move_angle / 2 / 10, 1650,
                          1500, 1650,
                          1500 + 1 * move_angle / 2 / 10, 1650,
                          1500 + 2 * move_angle / 2 / 10, 1650,
                          1500 + 3 * move_angle / 2 / 10, 1650,
                          1500 + 4 * move_angle / 2 / 10, 1650,
                          1500 + 5 * move_angle / 2 / 10, 1650,
                          1500 + 6 * move_angle / 2 / 10, 1650,
                          1500 + 7 * move_angle / 2 / 10, 1650,
                          1500 + 8 * move_angle / 2 / 10, 1650,
                          1500 + 9 * move_angle / 2 / 10, 1650,
                          1500 + 10 * move_angle / 2 / 10, 1500
                         };
leg CW_kick_frame[21] = {1500 + 10 * move_angle / 2 / 10, 1500, //kick forward with touching ground
                         1500 + 9 * move_angle / 2 / 10, 1550,
                         1500 + 8 * move_angle / 2 / 10, 1600,
                         1500 + 7 * move_angle / 2 / 10, 1650,
                         1500 + 6 * move_angle / 2 / 10, 1650,
                         1500 + 5 * move_angle / 2 / 10, 1650,
                         1500 + 4 * move_angle / 2 / 10, 1650,
                         1500 + 3 * move_angle / 2 / 10, 1650,
                         1500 + 2 * move_angle / 2 / 10, 1650,
                         1500 + 1 * move_angle / 2 / 10, 1650,
                         1500, 1650,
                         1500 - 1 * move_angle / 2 / 10, 1650,
                         1500 - 2 * move_angle / 2 / 10, 1650,
                         1500 - 3 * move_angle / 2 / 10, 1650,
                         1500 - 4 * move_angle / 2 / 10, 1650,
                         1500 - 5 * move_angle / 2 / 10, 1650,
                         1500 - 6 * move_angle / 2 / 10, 1650,
                         1500 - 7 * move_angle / 2 / 10, 1650,
                         1500 - 8 * move_angle / 2 / 10, 1650,
                         1500 - 9 * move_angle / 2 / 10, 1650,
                         1500 - 10 * move_angle / 2 / 10, 1500
                        };

void setup() {
  s1.attach(2);
  delay(200);
  s2.attach(3);
  delay(200);
  s3.attach(4);
  delay(200);
  s4.attach(5);
  delay(200);
  s5.attach(6);
  delay(200);
  s11.attach(9);
  delay(200);
  s12.attach(10);
  delay(200);
  s13.attach(11);
  delay(200);
  s14.attach(12);
  delay(200);
  s15.attach(13);
  delay(200);
  leg_position(1, 0);
  delay(200);
  leg_position(2, 1);
  delay(200);
  leg_position(3, 0);
  delay(200);
  leg_position(4, 1);
  delay(200);
  leg_position(5, 0);
  delay(3000);

  MsTimer2::set(75, walk); // 75ms period
  MsTimer2::start();
}

void loop() {


}

void walk() {
  if (command == 0) { //idle
    command = next_command();
  }
  else if (command == 1) {//forward
    if (state == 0) {
      leg_fine_control(1, CCW_kick_frame[0].leg_angle_us, CCW_kick_frame[0].leg_length_us);
      leg_fine_control(5, CCW_kick_frame[0].leg_angle_us, CCW_kick_frame[0].leg_length_us);
      leg_fine_control(2, CCW_stride_frame[0].leg_angle_us, CCW_stride_frame[0].leg_length_us);
      leg_fine_control(4, CCW_stride_frame[0].leg_angle_us, CCW_stride_frame[0].leg_length_us);
      leg_fine_control(3, CW_stride_frame[frame].leg_angle_us, CW_stride_frame[frame].leg_length_us);
    }
    else if (state == 1) {
      leg_fine_control(2, CCW_stride_frame[frame].leg_angle_us, CCW_stride_frame[frame].leg_length_us);
      leg_fine_control(4, CCW_stride_frame[frame].leg_angle_us, CCW_stride_frame[frame].leg_length_us);
      leg_fine_control(1, CCW_kick_frame[0].leg_angle_us, CCW_kick_frame[0].leg_length_us);
      leg_fine_control(5, CCW_kick_frame[0].leg_angle_us, CCW_kick_frame[0].leg_length_us);
      leg_fine_control(3, CCW_stride_frame[0].leg_angle_us, CCW_stride_frame[0].leg_length_us);
    }
    else if (state == 2) {
      leg_fine_control(2, CCW_kick_frame[0].leg_angle_us, CCW_kick_frame[0].leg_length_us);
      leg_fine_control(4, CCW_kick_frame[0].leg_angle_us, CCW_kick_frame[0].leg_length_us);
      leg_fine_control(1, CCW_kick_frame[frame].leg_angle_us, CCW_kick_frame[frame].leg_length_us);
      leg_fine_control(5, CCW_kick_frame[frame].leg_angle_us, CCW_kick_frame[frame].leg_length_us);
      leg_fine_control(3, CCW_stride_frame[0].leg_angle_us, CCW_stride_frame[0].leg_length_us);
    }
    else if (state == 3) {
      leg_fine_control(2, CCW_kick_frame[0].leg_angle_us, CCW_kick_frame[0].leg_length_us);
      leg_fine_control(4, CCW_kick_frame[0].leg_angle_us, CCW_kick_frame[0].leg_length_us);
      leg_fine_control(1, CCW_stride_frame[0].leg_angle_us, CCW_stride_frame[0].leg_length_us);
      leg_fine_control(5, CCW_stride_frame[0].leg_angle_us, CCW_stride_frame[0].leg_length_us);
      leg_fine_control(3, CCW_stride_frame[frame].leg_angle_us, CCW_stride_frame[frame].leg_length_us);
    }
    else if (state == 4) {
      leg_fine_control(2, CCW_kick_frame[frame].leg_angle_us, CCW_kick_frame[frame].leg_length_us);
      leg_fine_control(4, CCW_kick_frame[frame].leg_angle_us, CCW_kick_frame[frame].leg_length_us);
      leg_fine_control(1, CCW_stride_frame[0].leg_angle_us, CCW_stride_frame[0].leg_length_us);
      leg_fine_control(5, CCW_stride_frame[0].leg_angle_us, CCW_stride_frame[0].leg_length_us);
      leg_fine_control(3, CW_stride_frame[0].leg_angle_us, CW_stride_frame[0].leg_length_us);
    }
    else {
      leg_fine_control(2, CCW_stride_frame[0].leg_angle_us, CCW_stride_frame[0].leg_length_us);
      leg_fine_control(4, CCW_stride_frame[0].leg_angle_us, CCW_stride_frame[0].leg_length_us);
      leg_fine_control(1, CCW_stride_frame[frame].leg_angle_us, CCW_stride_frame[frame].leg_length_us);
      leg_fine_control(5, CCW_stride_frame[frame].leg_angle_us, CCW_stride_frame[frame].leg_length_us);
      leg_fine_control(3, CW_stride_frame[0].leg_angle_us, CW_stride_frame[0].leg_length_us);
    }
    frame++;

    if (frame >= 21)
    {
      frame = 0;
      state++;
      if (state >= 6) {
        state = 0;
        command = next_command();
      }
    }

  }
  else if (command == 2) {//backward
    if (state == 0) {
      leg_fine_control(1, CW_stride_frame[frame].leg_angle_us, CW_stride_frame[frame].leg_length_us);
      leg_fine_control(5, CW_stride_frame[frame].leg_angle_us, CW_stride_frame[frame].leg_length_us);
      leg_fine_control(2, CW_kick_frame[0].leg_angle_us, CW_kick_frame[0].leg_length_us);
      leg_fine_control(4, CW_kick_frame[0].leg_angle_us, CW_kick_frame[0].leg_length_us);
      leg_fine_control(3, CW_stride_frame[0].leg_angle_us, CW_stride_frame[0].leg_length_us);
    }
    else if (state == 1) {
      leg_fine_control(2, CW_kick_frame[frame].leg_angle_us, CW_kick_frame[frame].leg_length_us);
      leg_fine_control(4, CW_kick_frame[frame].leg_angle_us, CW_kick_frame[frame].leg_length_us);
      leg_fine_control(1, CW_kick_frame[0].leg_angle_us, CW_kick_frame[0].leg_length_us);
      leg_fine_control(5, CW_kick_frame[0].leg_angle_us, CW_kick_frame[0].leg_length_us);
      leg_fine_control(3, CW_stride_frame[0].leg_angle_us, CW_stride_frame[0].leg_length_us);
    }
    else if (state == 2) {
      leg_fine_control(2, CW_stride_frame[0].leg_angle_us, CW_stride_frame[0].leg_length_us);
      leg_fine_control(4, CW_stride_frame[0].leg_angle_us, CW_stride_frame[0].leg_length_us);
      leg_fine_control(1, CW_kick_frame[0].leg_angle_us, CW_kick_frame[0].leg_length_us);
      leg_fine_control(5, CW_kick_frame[0].leg_angle_us, CW_kick_frame[0].leg_length_us);
      leg_fine_control(3, CW_stride_frame[frame].leg_angle_us, CW_stride_frame[frame].leg_length_us);
    }
    else if (state == 3) {
      leg_fine_control(2, CW_stride_frame[0].leg_angle_us, CW_stride_frame[0].leg_length_us);
      leg_fine_control(4, CW_stride_frame[0].leg_angle_us, CW_stride_frame[0].leg_length_us);
      leg_fine_control(1, CW_kick_frame[frame].leg_angle_us, CW_kick_frame[frame].leg_length_us);
      leg_fine_control(5, CW_kick_frame[frame].leg_angle_us, CW_kick_frame[frame].leg_length_us);
      leg_fine_control(3, CCW_stride_frame[0].leg_angle_us, CCW_stride_frame[0].leg_length_us);
    }
    else if (state == 4) {
      leg_fine_control(2, CW_stride_frame[frame].leg_angle_us, CW_stride_frame[frame].leg_length_us);
      leg_fine_control(4, CW_stride_frame[frame].leg_angle_us, CW_stride_frame[frame].leg_length_us);
      leg_fine_control(1, CW_stride_frame[0].leg_angle_us, CW_stride_frame[0].leg_length_us);
      leg_fine_control(5, CW_stride_frame[0].leg_angle_us, CW_stride_frame[0].leg_length_us);
      leg_fine_control(3, CCW_stride_frame[0].leg_angle_us, CCW_stride_frame[0].leg_length_us);
    }
    else {
      leg_fine_control(2, CW_kick_frame[0].leg_angle_us, CW_kick_frame[0].leg_length_us);
      leg_fine_control(4, CW_kick_frame[0].leg_angle_us, CW_kick_frame[0].leg_length_us);
      leg_fine_control(1, CW_stride_frame[0].leg_angle_us, CW_stride_frame[0].leg_length_us);
      leg_fine_control(5, CW_stride_frame[0].leg_angle_us, CW_stride_frame[0].leg_length_us);
      leg_fine_control(3, CCW_stride_frame[frame].leg_angle_us, CCW_stride_frame[frame].leg_length_us);
    }
    frame++;
    if (frame >= 21)
    {
      frame = 0;
      state++;
      if (state >= 6) {
        state = 0;
        command = next_command();
      }
    }
  }
  else if (command == 3) {//left turn
    if (state == 0) {
      leg_fine_control(1, CW_stride_frame[0].leg_angle_us, CW_stride_frame[0].leg_length_us);
      leg_fine_control(2, CW_kick_frame[0].leg_angle_us, CW_kick_frame[0].leg_length_us);
      leg_fine_control(3, CW_stride_frame[0].leg_angle_us, CW_stride_frame[0].leg_length_us);
      leg_fine_control(4, CCW_stride_frame[0].leg_angle_us, CCW_stride_frame[0].leg_length_us);
      leg_fine_control(5, CCW_kick_frame[frame].leg_angle_us, CCW_kick_frame[frame].leg_length_us);
    }
    else if (state == 1) {
      leg_fine_control(1, CW_stride_frame[0].leg_angle_us, CW_stride_frame[0].leg_length_us);
      leg_fine_control(2, CW_kick_frame[0].leg_angle_us, CW_kick_frame[0].leg_length_us);
      leg_fine_control(3, CW_stride_frame[0].leg_angle_us, CW_stride_frame[0].leg_length_us);
      leg_fine_control(4, CCW_stride_frame[frame].leg_angle_us, CCW_stride_frame[frame].leg_length_us);
      leg_fine_control(5, CCW_stride_frame[0].leg_angle_us, CCW_stride_frame[0].leg_length_us);
    }
    else if (state == 2) {
      leg_fine_control(1, CW_stride_frame[0].leg_angle_us, CW_stride_frame[0].leg_length_us);
      leg_fine_control(2, CW_kick_frame[0].leg_angle_us, CW_kick_frame[0].leg_length_us);
      leg_fine_control(3, CW_stride_frame[0].leg_angle_us, CW_stride_frame[0].leg_length_us);
      leg_fine_control(4, CCW_kick_frame[frame].leg_angle_us, CCW_kick_frame[frame].leg_length_us);
      leg_fine_control(5, CCW_stride_frame[0].leg_angle_us, CCW_stride_frame[0].leg_length_us);
    }
    else if (state == 3) {
      leg_fine_control(1, CW_stride_frame[0].leg_angle_us, CW_stride_frame[0].leg_length_us);
      leg_fine_control(2, CW_kick_frame[0].leg_angle_us, CW_kick_frame[0].leg_length_us);
      leg_fine_control(3, CW_stride_frame[0].leg_angle_us, CW_stride_frame[0].leg_length_us);
      leg_fine_control(4, CCW_stride_frame[0].leg_angle_us, CCW_stride_frame[0].leg_length_us);
      leg_fine_control(5, CCW_stride_frame[frame].leg_angle_us, CCW_stride_frame[frame].leg_length_us);
    }
    else if (state == 4) {
      leg_fine_control(1, CW_stride_frame[frame].leg_angle_us, CW_stride_frame[frame].leg_length_us);
      leg_fine_control(2, CW_kick_frame[0].leg_angle_us, CW_kick_frame[0].leg_length_us);
      leg_fine_control(3, CW_stride_frame[0].leg_angle_us, CW_stride_frame[0].leg_length_us);
      leg_fine_control(4, CCW_stride_frame[0].leg_angle_us, CCW_stride_frame[0].leg_length_us);
      leg_fine_control(5, CCW_kick_frame[0].leg_angle_us, CCW_kick_frame[0].leg_length_us);
    }
    else if (state == 5) {
      leg_fine_control(1, CW_kick_frame[0].leg_angle_us, CW_kick_frame[0].leg_length_us);
      leg_fine_control(2, CW_kick_frame[frame].leg_angle_us, CW_kick_frame[frame].leg_length_us);
      leg_fine_control(3, CW_stride_frame[0].leg_angle_us, CW_stride_frame[0].leg_length_us);
      leg_fine_control(4, CCW_stride_frame[0].leg_angle_us, CCW_stride_frame[0].leg_length_us);
      leg_fine_control(5, CCW_kick_frame[0].leg_angle_us, CCW_kick_frame[0].leg_length_us);
    }
    else if (state == 6) {
      leg_fine_control(1, CW_kick_frame[0].leg_angle_us, CW_kick_frame[0].leg_length_us);
      leg_fine_control(2, CW_stride_frame[frame].leg_angle_us, CW_stride_frame[frame].leg_length_us);
      leg_fine_control(3, CW_stride_frame[0].leg_angle_us, CW_stride_frame[0].leg_length_us);
      leg_fine_control(4, CCW_stride_frame[0].leg_angle_us, CCW_stride_frame[0].leg_length_us);
      leg_fine_control(5, CCW_kick_frame[0].leg_angle_us, CCW_kick_frame[0].leg_length_us);
    }

    else {
      leg_fine_control(1, CW_kick_frame[frame].leg_angle_us, CW_kick_frame[frame].leg_length_us);
      leg_fine_control(2, CW_kick_frame[0].leg_angle_us, CW_kick_frame[0].leg_length_us);
      leg_fine_control(3, CW_stride_frame[0].leg_angle_us, CW_stride_frame[0].leg_length_us);
      leg_fine_control(4, CCW_stride_frame[0].leg_angle_us, CCW_stride_frame[0].leg_length_us);
      leg_fine_control(5, CCW_kick_frame[0].leg_angle_us, CCW_kick_frame[0].leg_length_us);
    }
    frame++;
    if (frame >= 21)
    {
      frame = 0;
      state++;
      if (state >= 8) {
        state = 0;
        command = next_command();
      }
    }
  }
  else if (command == 4) {//right turn
    if (state == 0) {
      leg_fine_control(1, CCW_kick_frame[0].leg_angle_us, CCW_kick_frame[0].leg_length_us);
      leg_fine_control(2, CCW_stride_frame[0].leg_angle_us, CCW_stride_frame[0].leg_length_us);
      leg_fine_control(3, CCW_kick_frame[0].leg_angle_us, CCW_kick_frame[0].leg_length_us);
      leg_fine_control(4, CW_kick_frame[0].leg_angle_us, CW_kick_frame[0].leg_length_us);
      leg_fine_control(5, CW_stride_frame[frame].leg_angle_us, CW_stride_frame[frame].leg_length_us);
    }
    else if (state == 1) {
      leg_fine_control(1, CCW_kick_frame[0].leg_angle_us, CCW_kick_frame[0].leg_length_us);
      leg_fine_control(2, CCW_stride_frame[0].leg_angle_us, CCW_stride_frame[0].leg_length_us);
      leg_fine_control(3, CCW_kick_frame[0].leg_angle_us, CCW_kick_frame[0].leg_length_us);
      leg_fine_control(4, CW_kick_frame[frame].leg_angle_us, CW_kick_frame[frame].leg_length_us);
      leg_fine_control(5, CW_kick_frame[0].leg_angle_us, CW_kick_frame[0].leg_length_us);
    }
    else if (state == 2) {
      leg_fine_control(1, CCW_kick_frame[0].leg_angle_us, CCW_kick_frame[0].leg_length_us);
      leg_fine_control(2, CCW_stride_frame[0].leg_angle_us, CCW_stride_frame[0].leg_length_us);
      leg_fine_control(3, CCW_kick_frame[0].leg_angle_us, CCW_kick_frame[0].leg_length_us);
      leg_fine_control(4, CW_stride_frame[frame].leg_angle_us, CW_stride_frame[frame].leg_length_us);
      leg_fine_control(5, CW_kick_frame[0].leg_angle_us, CW_kick_frame[0].leg_length_us);
    }
    else if (state == 3) {
      leg_fine_control(1, CCW_kick_frame[0].leg_angle_us, CCW_kick_frame[0].leg_length_us);
      leg_fine_control(2, CCW_stride_frame[0].leg_angle_us, CCW_stride_frame[0].leg_length_us);
      leg_fine_control(3, CCW_kick_frame[0].leg_angle_us, CCW_kick_frame[0].leg_length_us);
      leg_fine_control(4, CW_kick_frame[0].leg_angle_us, CW_kick_frame[0].leg_length_us);
      leg_fine_control(5, CW_kick_frame[frame].leg_angle_us, CW_kick_frame[frame].leg_length_us);
    }
    else if (state == 4) {
      leg_fine_control(1, CCW_kick_frame[frame].leg_angle_us, CCW_kick_frame[frame].leg_length_us);
      leg_fine_control(2, CCW_stride_frame[0].leg_angle_us, CCW_stride_frame[0].leg_length_us);
      leg_fine_control(3, CCW_kick_frame[0].leg_angle_us, CCW_kick_frame[0].leg_length_us);
      leg_fine_control(4, CW_kick_frame[0].leg_angle_us, CW_kick_frame[0].leg_length_us);
      leg_fine_control(5, CW_stride_frame[0].leg_angle_us, CW_stride_frame[0].leg_length_us);
    }
    else if (state == 5) {
      leg_fine_control(1, CCW_stride_frame[0].leg_angle_us, CCW_stride_frame[0].leg_length_us);
      leg_fine_control(2, CCW_stride_frame[frame].leg_angle_us, CCW_stride_frame[frame].leg_length_us);
      leg_fine_control(3, CCW_kick_frame[0].leg_angle_us, CCW_kick_frame[0].leg_length_us);
      leg_fine_control(4, CW_kick_frame[0].leg_angle_us, CW_kick_frame[0].leg_length_us);
      leg_fine_control(5, CW_stride_frame[0].leg_angle_us, CW_stride_frame[0].leg_length_us);
    }
    else if (state == 6) {
      leg_fine_control(1, CCW_stride_frame[0].leg_angle_us, CCW_stride_frame[0].leg_length_us);
      leg_fine_control(2, CCW_kick_frame[frame].leg_angle_us, CCW_kick_frame[frame].leg_length_us);
      leg_fine_control(3, CCW_kick_frame[0].leg_angle_us, CCW_kick_frame[0].leg_length_us);
      leg_fine_control(4, CW_kick_frame[0].leg_angle_us, CW_kick_frame[0].leg_length_us);
      leg_fine_control(5, CW_stride_frame[0].leg_angle_us, CW_stride_frame[0].leg_length_us);
    }

    else {
      leg_fine_control(1, CCW_stride_frame[frame].leg_angle_us, CCW_stride_frame[frame].leg_length_us);
      leg_fine_control(2, CCW_stride_frame[0].leg_angle_us, CCW_stride_frame[0].leg_length_us);
      leg_fine_control(3, CCW_kick_frame[0].leg_angle_us, CCW_kick_frame[0].leg_length_us);
      leg_fine_control(4, CW_kick_frame[0].leg_angle_us, CW_kick_frame[0].leg_length_us);
      leg_fine_control(5, CW_stride_frame[0].leg_angle_us, CW_stride_frame[0].leg_length_us);
    }
    frame++;
    if (frame >= 21)
    {
      frame = 0;
      state++;
      if (state >= 8) {
        state = 0;
        command = next_command();
      }
    }
  }
  else { //wrong state, get new command
    command = next_command();
  }

}

void leg_position(int leg_num, int leg_pos) {
  switch (leg_num) {
    case 1:
      if (leg_pos == 0) {
        s1.writeMicroseconds(1200 + servo_angle_offset[0]);
        s11.writeMicroseconds(1500 + servo_length_offset[0]);
      }
      else if (leg_pos == 1) {
        s1.writeMicroseconds(1780 + servo_angle_offset[0]);
        s11.writeMicroseconds(1500 + servo_length_offset[0]);
      }
      else if (leg_pos == 2) {
        s1.writeMicroseconds(1780 + servo_angle_offset[0]);
        s11.writeMicroseconds(1999 + servo_length_offset[0]);
      }
      else if (leg_pos == 3) {
        s1.writeMicroseconds(1200 + servo_angle_offset[0]);
        s11.writeMicroseconds(1999 + servo_length_offset[0]);
      }
      else {
        s1.writeMicroseconds(1500 + servo_angle_offset[0]);
        s11.writeMicroseconds(1500 + servo_length_offset[0]);
      }
      break;
    case 2:
      if (leg_pos == 0) {
        s2.writeMicroseconds(1200 + servo_angle_offset[1]);
        s12.writeMicroseconds(1500 + servo_length_offset[1]);
      }
      else if (leg_pos == 1) {
        s2.writeMicroseconds(1780 + servo_angle_offset[1]);
        s12.writeMicroseconds(1500 + servo_length_offset[1]);
      }
      else if (leg_pos == 2) {
        s2.writeMicroseconds(1780 + servo_angle_offset[1]);
        s12.writeMicroseconds(1999 + servo_length_offset[1]);
      }
      else if (leg_pos == 3) {
        s2.writeMicroseconds(1200 + servo_angle_offset[1]);
        s12.writeMicroseconds(1999 + servo_length_offset[1]);
      }
      else {
        s2.writeMicroseconds(1500 + servo_angle_offset[1]);
        s12.writeMicroseconds(1500 + servo_length_offset[1]);
      }
      break;
    case 3:
      if (leg_pos == 0) {
        s3.writeMicroseconds(1200 + servo_angle_offset[2]);
        s13.writeMicroseconds(1500 + servo_length_offset[2]);
      }
      else if (leg_pos == 1) {
        s3.writeMicroseconds(1780 + servo_angle_offset[2]);
        s13.writeMicroseconds(1500 + servo_length_offset[2]);
      }
      else if (leg_pos == 2) {
        s3.writeMicroseconds(1780 + servo_angle_offset[2]);
        s13.writeMicroseconds(1999 + servo_length_offset[2]);
      }
      else if (leg_pos == 3) {
        s3.writeMicroseconds(1200 + servo_angle_offset[2]);
        s13.writeMicroseconds(1999 + servo_length_offset[2]);
      }
      else {
        s3.writeMicroseconds(1500 + servo_angle_offset[2]);
        s13.writeMicroseconds(1500 + servo_length_offset[2]);
      }
      break;
    case 4:
      if (leg_pos == 0) {
        s4.writeMicroseconds(1200 + servo_angle_offset[3]);
        s14.writeMicroseconds(1500 + servo_length_offset[3]);
      }
      else if (leg_pos == 1) {
        s4.writeMicroseconds(1780 + servo_angle_offset[3]);
        s14.writeMicroseconds(1500 + servo_length_offset[3]);
      }
      else if (leg_pos == 2) {
        s4.writeMicroseconds(1780 + servo_angle_offset[3]);
        s14.writeMicroseconds(1999 + servo_length_offset[3]);
      }
      else if (leg_pos == 3) {
        s4.writeMicroseconds(1200 + servo_angle_offset[3]);
        s14.writeMicroseconds(1999 + servo_length_offset[3]);
      }
      else {
        s4.writeMicroseconds(1500 + servo_angle_offset[3]);
        s14.writeMicroseconds(1500 + servo_length_offset[3]);
      }
      break;
    case 5:
      if (leg_pos == 0) {
        s5.writeMicroseconds(1200 + servo_angle_offset[4]);
        s15.writeMicroseconds(1500 + servo_length_offset[4]);
      }
      else if (leg_pos == 1) {
        s5.writeMicroseconds(1780 + servo_angle_offset[4]);
        s15.writeMicroseconds(1500 + servo_length_offset[4]);
      }
      else if (leg_pos == 2) {
        s5.writeMicroseconds(1780 + servo_angle_offset[4]);
        s15.writeMicroseconds(1999 + servo_length_offset[4]);
      }
      else if (leg_pos == 3) {
        s5.writeMicroseconds(1200 + servo_angle_offset[4]);
        s15.writeMicroseconds(1999 + servo_length_offset[4]);
      }
      else {
        s5.writeMicroseconds(1500 + servo_angle_offset[4]);
        s15.writeMicroseconds(1500 + servo_length_offset[4]);
      }
      break;
    default:
      break;
  }
}
void leg_fine_control(int leg_num, int leg_angle_us, int leg_length_us) {
  switch (leg_num) {
    case 1:
      s1.writeMicroseconds(leg_angle_us + servo_angle_offset[0]);
      s11.writeMicroseconds(leg_length_us + servo_length_offset[0]);
      break;
    case 2:
      s2.writeMicroseconds(leg_angle_us + servo_angle_offset[1]);
      s12.writeMicroseconds(leg_length_us + servo_length_offset[1]);
      break;
    case 3:
      s3.writeMicroseconds(leg_angle_us + servo_angle_offset[2]);
      s13.writeMicroseconds(leg_length_us + servo_length_offset[2]);
      break;
    case 4:
      s4.writeMicroseconds(leg_angle_us + servo_angle_offset[3]);
      s14.writeMicroseconds(leg_length_us + servo_length_offset[3]);
      break;
    case 5:
      s5.writeMicroseconds(leg_angle_us + servo_angle_offset[4]);
      s15.writeMicroseconds(leg_length_us + servo_length_offset[4]);
      break;
    default:
      break;
  }
}

int next_command() {
  int next_command = 0;
  boolean L_clear, R_clear;
  if (analogRead(A0) < 200)
  {
    R_clear = true;
  }
  else
  {
    R_clear = false;
  }

  if (analogRead(A1) < 200)
  {
    L_clear = true;
  }
  else
  {
    L_clear = false;
  }

  if (L_clear && R_clear)
  {
    next_command = 1;
  }
  else if (L_clear && !R_clear)
  {
    next_command = 3;
  }
  else if (!L_clear && R_clear)
  {
    next_command = 4;
  }
  else
  {
    next_command = 2;
  }
  return next_command;



}
